class FolderNotFoundException(Exception):
    """
    Exception displayed when the static folder not exists.
    """
    pass


class SemanticUIException(Exception):
    """
    Exception displayed during the install/uninstall/gulp build commands
    """
    pass


from django.core.management.base import BaseCommand
from django_semantic_ui.models import SemanticUI


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('subcommand', choices=['install', 'uninstall', 'gulp_build'])

    def handle(self, *args, **options):
        if 'subcommand' in options:
            object = SemanticUI()
            if options.get('subcommand') == 'install':
                object.install()
            elif options.get('subcommand') == 'uninstall':
                object.uninstall()
            elif options.get('subcommand') == 'gulp_build':
                object.build()
            else:
                print(self.help)



from django.utils.html import format_html
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe
from django.utils.encoding import force_text
from django_semantic_ui.exceptions import SemanticUIException
from django.conf import settings


class Constant:
    """
    Default Constant used on the DSU package.
    """
    JQUERY_URL = 'https://code.jquery.com/jquery-3.1.1.min.js'
    JQUERY_INTEGRITY_HASH = 'sha256-hVVnYaiADRTO2PzUGmuLJr8BLUSjGIZsDYGmIJLv2b8='
    JQUERY_CROSSORIGIN = 'anonymous'
    GULP_VERSION = '3.9.1'
    SEMANTIC_UI_VERSION = 'latest'
    SEMANTIC_DIRNAME = 'semantic'
    JAVASCRIPT_FILE = '/dist/semantic.min.js'
    STYLESHEET_FILE = '/dist/semantic.min.css'
    PACKAGE_NAME = 'dsu'
    STATIC_URL = '/static/'

    def __init__(self):
        pass


class Utils:
    """
    Default static methods used on the DSU package.
    """

    def __init__(self):
        pass

    @staticmethod
    def get_static_folder():
        """
        Return the STATIC url defined on the django project
        :return: STATIC url defined.
        """
        try:
            static_url = settings.STATIC_URL
        except:
            static_url = Constant.STATIC_URL
        return static_url

    @staticmethod
    def get_document_url(semantic_folder=Constant.SEMANTIC_DIRNAME, filename=None):
        """
        Get the relative path from any semantic file
        :param semantic_folder: the absolute semantic folder path
        :param filename: the filename selected
        :return: relative path from the semantic file selected.
        """
        if not filename:
            raise SemanticUIException(
                "[ERROR] The filename  from javascript, component or stylesheet is required.")
        else:
            return '{0}{1}/{2}{3}'.format(
                Utils.get_static_folder(), Constant.PACKAGE_NAME,
                semantic_folder, filename)

    @staticmethod
    def get_text_value(value=None):
        """
        Format the HTML content inside from HTML tag selected.
        :param value: HTML content unformatted inside from a HTML tag.
        :return: HTML content formatted
        """
        return force_text(value) if value else ''

    @staticmethod
    def get_attrs_formatted(attrs=None):
        """
        Format all attrs defined on a HTML tag selected.
        :param attrs: dictionary with all attrs defined in a HTML tag.
        :return: Attrs formatted.
        """
        return mark_safe(flatatt(attrs=attrs)) if attrs else ''

    @staticmethod
    def render_tag(tag, attrs=None, content=None, close=True):
        """
        Build a HTML tag with attributes, HTML content.
        :param tag: HTML tag name
        :param attrs: dictionary with all attrs defined on the HTML tag.
        :param content: HTML content added inside HTML tag.
        :param close: boolean to close HTML tag.
        :return: the HTML tag rendered.
        """
        content_formatted = Utils.get_text_value(value=content)
        attrs_formatted = Utils.get_attrs_formatted(attrs=attrs)
        html = "<{0} {1}>{2}".format(tag, attrs_formatted, content_formatted)
        html += "</{0}>".format(tag) if close or content else ""
        return format_html(
            format_string=html,
            tag=tag,
            attrs=attrs_formatted,
            content=content_formatted)



from django import template
from django_semantic_ui.utils import Utils, Constant
from django_semantic_ui.models import SemanticUI


register = template.Library()
dsu = SemanticUI()


@register.simple_tag()
def dsu_javascript_url():
    """
    Template tag that allow load the main semantic-ui javascript lib.
    example to use
    {% dsu_javascript_url %}
    :return: string
    """
    return Utils.render_tag(
        tag='script',
        attrs={
            'src': Utils.get_document_url(
                semantic_folder=dsu.semantic_folder, filename=Constant.JAVASCRIPT_FILE)
        })


@register.simple_tag()
def dsu_stylesheet_url():
    """
    Template tag that allow load the main semantic-ui stylesheet.
    example to use
    {% dsu_stylesheet_url %}
    :return:
    """
    return Utils.render_tag(
        tag='link',
        attrs={
            'href': Utils.get_document_url(
                semantic_folder=dsu.semantic_folder, filename=Constant.STYLESHEET_FILE),
            'rel': 'stylesheet',
            'type': 'text/css'}, close=False)


@register.simple_tag()
def dsu_jquery_url():
    """
    Template tag that allow load the default jquery javascript lib required by semantic-ui (this tag is optional)
    example to use
    {% dsu_jquery_url %}
    NOTE: You can change the jquery_url in your settings.py using the DSU_JQUERY_URL setting.
    :return: string
    """
    options = {'src': dsu.jquery_url}
    if Constant.JQUERY_URL == dsu.jquery_url:
        options = {
            'src': dsu.jquery_url,
            'integrity': Constant.JQUERY_INTEGRITY_HASH,
            'crossorigin': Constant.JQUERY_CROSSORIGIN}
    return Utils.render_tag(tag='script', attrs=options)



import os
import time
from django.conf import settings
from .exceptions import FolderNotFoundException, SemanticUIException
from .utils import Constant


class SemanticUI(object):
    """
    Main class used to install, gulp build and uninstall the Semantic UI Framework.
    """
    SECONDS_WAITING_ALLOWED = 5

    def __init__(self):
        """
        Build the default folder required by the DSU package.
        Define the default global variables required by the DSU package.
        """
        self.project_folder_path = os.path.dirname(__file__)
        try:
            self.jquery_url = settings.DSU_JQUERY_URL
        except:
            self.jquery_url = Constant.JQUERY_URL
        try:
            self.gulp_version = settings.GULP_VERSION
        except:
            self.gulp_version = Constant.GULP_VERSION
        try:
            self.semantic_ui_version = settings.SEMANTIC_UI_VERSION
        except:
            self.semantic_ui_version = Constant.SEMANTIC_UI_VERSION
        try:
            self.semantic_folder = settings.SEMANTIC_DIRNAME
        except:
            self.semantic_folder = Constant.SEMANTIC_DIRNAME

    def move_static_folder(self):
        try:
            if not os.path.exists(self.project_folder_path):
                raise FolderNotFoundException("[ERROR] Project folder not foound. PATH: {0}".format(
                    self.project_folder_path))
            else:
                self.static_folder_path = '{0}/static'.format(self.project_folder_path)
                if not os.path.exists(self.static_folder_path):
                    print("Creating the Static folder in the PATH: {0}".format(
                        self.static_folder_path))
                    os.mkdir(self.static_folder_path)
                try:
                    os.chdir(self.static_folder_path)
                    self.semantic_files_path = '{0}/dsu'.format(self.static_folder_path)
                    if not os.path.exists(self.semantic_files_path):
                        print("Creating the django_semantic_ui folder on PATH: {0}".format(
                            self.semantic_files_path))
                        os.mkdir(self.semantic_files_path)
                    print("Semantic UI Folder: {0}".format(self.semantic_files_path))
                except:
                    raise FolderNotFoundException("[ERROR] Static folder not found")
        except Exception as e:
            raise SemanticUIException(e)

    def install(self):
        """
        Install Gulp and Semantic UI Framework on a django project
        :return: None
        """
        print("Installing started...")
        self.move_static_folder()
        try:
            print("Moving to static folder...")
            os.chdir(self.semantic_files_path)
            package_json_path = '{0}/package.json'.format(self.static_folder_path)
            file_exists = os.path.isfile(package_json_path)
            if not file_exists:
                print("Generationg the package.json file...")
                os.system('npm init')
                print("Package.json file has been generated successfully!.")
            print("Installing gulp...")
            os.system('npm install gulp@{0} --save-dev'.format(self.gulp_version))
            print("Gulp module has been installed successfully (Version installed {0})".format(
                self.gulp_version))
            print("Installing semantic-ui module...")
            os.system('npm install semantic-ui@{0} --save'.format(self.semantic_ui_version))
            print("Semantic-ui module has been installed successfully (Version installed {0}).".format(
                self.semantic_ui_version))
            print('Indexing "semantic-ui" on your django project path: {0}'.format(settings.BASE_DIR))
            print("Indexing modules...")
            time.sleep(self.SECONDS_WAITING_ALLOWED)
            os.chdir(settings.BASE_DIR)
            time.sleep(5)
            print("Semantic UI  has been installed successfully...")
        except:
            raise SemanticUIException("[ERROR] Installing failed cannot move to static folder.")

    def uninstall(self):
        """
        Uninstall Gulp and Semantic UI framework from the django project.
        :return: None
        """
        print("Uninstalling started...")
        self.move_static_folder()
        try:
            print("Moving to Static folder...")
            os.chdir(self.semantic_files_path)
            print("Uninstalling Semantic UI module...")
            os.system('npm uninstall semantic-ui --save')
            print("Semantic UI module has been removed successfully!.")
            print("Uninstalling Gulp module...")
            os.system('npm uninstall gulp --save-dev')
            print("Gulp module has been removed successfully!.")
            print("Deleting semantic folder...")
            try:
                os.system('rm -rf {0}'.format(self.semantic_folder))
                print("Semantic folder has been removed successfully.")
                print("Deleting semantic.json file...")
                try:
                    os.system('rm -f semantic.json')
                    print("semantic.json file has been removed...")
                    print("Deleting package.json file...")
                    try:
                        os.system("rm -rf package*")
                        print("package.json file has been removed.")
                        node_modules_path = '{0}/node_modules'.format(self.semantic_files_path)
                        if not os.path.exists(node_modules_path):
                            raise FolderNotFoundException("[ERROR] node_modules folder not found. PATH: {0}".format(
                                node_modules_path))
                        else:
                            os.system("rm -rf node_modules")
                        print("Removing semantic folder...")
                        os.rmdir(self.semantic_files_path)
                        print("Semantic folder removed.")
                        print("Removing static folder...")
                        os.rmdir(self.static_folder_path)
                        print("Static folder removed.")
                        print('Indexing "semantic-ui" on your django project path: {0}'.format(settings.BASE_DIR))
                        print("Indexing modules...")
                        time.sleep(self.SECONDS_WAITING_ALLOWED)
                        os.chdir(settings.BASE_DIR)
                        time.sleep(5)
                        print("Semantic UI has been removed successfully!")
                    except:
                        raise SemanticUIException("[ERROR] Semantic JSON file not exists.")
                except:
                    raise SemanticUIException("[ERROR] Semantic JSON file not exists.")
            except:
                raise SemanticUIException("[ERROR] Semantic folder not exists.")
        except:
            raise SemanticUIException("[ERROR] Uninstalling failed cannot move to static folder.")

    def build(self):
        """
        Running Gulp build command inside semantic folder.
        :return: None
        """
        print("Gulp Build command started...")
        self.move_static_folder()
        print("Moving to Static folder...")
        os.chdir(self.static_folder_path)
        semantic_path = '{0}/{1}'.format(self.semantic_files_path, self.semantic_folder)
        try:
            print("Moving to Semantic folder...")
            os.chdir(semantic_path)
            print("Gulp build command executing...")
            os.system('gulp build')
            print('Indexing "semantic-ui" on your django project path: {0}'.format(settings.BASE_DIR))
            print("Indexing modules...")
            time.sleep(self.SECONDS_WAITING_ALLOWED)
            os.chdir(settings.BASE_DIR)
            time.sleep(5)
            print("Gulp build command completed successfully.")
        except:
            raise SemanticUIException(
                "[ERROR] Gulp build command failed, the semantic folder not exists or the name is incorrect.")



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
    name="django2_semantic_ui",
    version="1.2.2",
    author="Franklin Sarmiento",
    author_email="franklinitiel@gmail.com",
    description="Library to easy install, configure and use Semantic UI framework with Django project (Python 3.x)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/franklintiel/django2-semantic-ui/wiki",
    license="MIT",
    classifiers=[
        #"Development Status :: 4 - Beta",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Natural Language :: English',
    ],
    keywords="semantic-ui semanticui semantic ui django-semantic-ui django-semanticui django-semantic-ui",
    project_urls={
        'Documentation': "https://github.com/franklintiel/django2-semantic-ui",
        'Source': "https://github.com/franklintiel/django2-semantic-ui",
        'Tracker': "https://github.com/franklintiel/django2-semantic-ui/issues"
    },
    packages=setuptools.find_packages(exclude=['contrib', 'docs', 'tests*']),
    python_requires=">=3.*")
